import json
import logging
from abc import ABC, abstractmethod
import math
from sys import exit
import re


def remove_ansi_escape_sequences(text):
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')

    return ansi_escape.sub('', text)


class MicroprintGenerator(ABC):
    def _load_config_file(self):
        config_file_path = self.config_file_path

        try:
            _file = open(config_file_path)
        except OSError as _:
            logging.error(
                f"Couldn't open config file '{config_file_path}'. Using default parameters.")
            self.rules = {}
        else:
            with _file:
                logging.info(
                    f"Configuration file '{config_file_path} loaded successfully")
                rules = json.load(_file)

                self.rules = rules

    def _set_default_colors(self):
        fallback_colors = {"background_color": "white", "text_color": "black"}

        default_colors = self.rules.get("default_colors", fallback_colors)

        default_colors["background_color"] = default_colors.get(
            "background_color", fallback_colors["background_color"])

        default_colors["text_color"] = default_colors.get(
            "text_color", fallback_colors["text_color"])

        self.default_colors = default_colors

    def __init__(self, output_filename, config_file_path, text):

        logging.getLogger().setLevel(logging.INFO)

        self.output_filename = output_filename

        self.text_lines = remove_ansi_escape_sequences(text).split('\n')

        self.config_file_path = config_file_path

        self._load_config_file()

        self.scale = self.rules.get("scale", 2)
        self.vertical_spacing = self.rules.get("vertical_spacing", 1)

        self.scale_with_spacing = self.scale * self.vertical_spacing

        self.scaled_microprint_height = len(
            self.text_lines) * self.scale_with_spacing

        if "number_of_columns" not in self.rules:

            self.max_microprint_height = self.rules.get(
                "max_microprint_height", len(self.text_lines) * self.scale_with_spacing)

            self.number_of_columns = math.ceil(
                self.scaled_microprint_height / self.max_microprint_height)
        else:
            self.number_of_columns = self.rules["number_of_columns"]

            self.max_microprint_height = math.floor(
                self.scaled_microprint_height / self.number_of_columns)

        self.column_width = self.rules.get(
            "microprint_width", 120)

        self.column_gap_size = self.rules.get(
            "column_gap_size", 0.2) * self.scale

        self.column_gap_color = self.rules.get("column_gap_color", "white")

        self.microprint_width = (
            self.column_width + self.column_gap_size) * self.number_of_columns

        self.microprint_height = min(
            len(self.text_lines) * self.scale_with_spacing, self.max_microprint_height)

        self.text_lines_per_column = math.floor(
            self.microprint_height / self.scale_with_spacing)

        self._set_default_colors()

    @classmethod
    def from_text_file(cls, output_filename="microprint.svg", config_file_path="config.json", file_path=""):
        try:
            _file = open(file_path)
        except OSError as _:
            exit(f"Couldn't open text file '{file_path}'. Aborting execution.")
        else:
            with file:
                text = file.read()

                return cls(output_filename=output_filename, config_file_path=config_file_path, text=text)

    def check_color_line_rule(self, color_type, text_line):
        text_line = text_line.lower()

        line_rules = self.rules.get("line_rules", {})

        default_color = self.default_colors[color_type]

        for rule in line_rules:
            try:
                pattern = re.compile(rule, re.IGNORECASE)
                if re.search(pattern, text_line):
                    return line_rules[rule].get(color_type, default_color)

            except re.error:
                if text_line.find(rule) != -1:
                    return line_rules[rule].get(color_type, default_color)

        return default_color

    @abstractmethod
    def render_microprint(self):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

import argparse
from .SVGMicroprintGenerator import SVGMicroprintGenerator


def main():
    parser = argparse.ArgumentParser(prog='μPrintGen',
                                     description='Microprint generator')

    parser.add_argument('-i', '--file_input',
                        required=True,
                        dest='file_input',
                        help="File path for the input text to generate the microprint.")
    parser.add_argument('-c', '--config_file_path',
                        dest='config_file_path',
                        help="Config file path for the .json file containing the configurations of the generated "
                             "microprint.",
                        default="config.json"
                        )
    parser.add_argument('-o', '--output_filename', dest='output_filename',
                        help='Output filename for the generated microprint',
                        default="Microprint.svg"
                        )



    args = parser.parse_args()

    SVGMicroprintGenerator.from_text_file(output_filename=args.output_filename,
                                          config_file_path=args.config_file_path,
                                          file_path=args.file_input).render_microprint()



import math
from .MicroprintGenerator import MicroprintGenerator
import svgwrite
import logging
from tqdm import tqdm


class SVGMicroprintGenerator(MicroprintGenerator):

    def _load_svg_fonts(self):

        additional_fonts = self.rules.get("additional_fonts",
                                          {"google_fonts": [],
                                           "truetype_fonts": []})

        google_fonts = additional_fonts.get("google_fonts", [])

        truetype_fonts = additional_fonts.get("truetype_fonts", [])

        for count, google_font in enumerate(google_fonts):
            name = google_font["name"]
            url = google_font["google_font_url"]

            self.drawing.embed_google_web_font(name, url)

        for count, truetype_font in enumerate(truetype_fonts):
            name = truetype_font["name"]
            truetype_file = truetype_font["truetype_file"]

            self.drawing.embed_font(name, truetype_file)

    def __init__(self, output_filename="microprint.svg", config_file_path="config.json", text=""):
        super().__init__(output_filename=output_filename,
                         config_file_path=config_file_path, text=text)

        self.drawing = svgwrite.Drawing(
            output_filename, (self.microprint_width, self.microprint_height), debug=False)

        self.font_family = self.rules.get("font-family", "monospace")

        self._load_svg_fonts()

    def render_microprint_column(self, first_line, last_line, x_with_gap, y, current_line):
        backgrounds = self.drawing.add(self.drawing.g())

        default_text_color = self.default_colors["text_color"]

        texts = self.drawing.add(self.drawing.g(
            font_size=self.scale, fill=default_text_color))

        attributes = {'xml:space': 'preserve',
                      "font-family": self.font_family}

        texts.update(attributes)

        text_lines = self.text_lines[first_line:last_line]

        for text_line in tqdm(text_lines, total=len(text_lines), desc="Generating rows"):
            background_color = self.check_color_line_rule(
                color_type="background_color", text_line=text_line)

            background_rect = self.drawing.rect(insert=(x_with_gap, y),
                                                size=(self.column_width,
                                                      self.scale + 0.3),
                                                rx=None, ry=None, fill=background_color)

            text_color = self.check_color_line_rule(
                color_type="text_color", text_line=text_line)

            text = self.drawing.text(text_line, insert=(x_with_gap, y),
                                     fill=text_color, dominant_baseline="hanging")

            text.update({"data-text-line": current_line})
            background_rect.update({"data-text-line": current_line})

            backgrounds.add(background_rect)
            texts.add(text)

            y += self.scale_with_spacing

            current_line += 1

    def render_microprint(self):
        logging.info('Generating svg microprint')

        default_background_color = self.default_colors["background_color"]

        self.drawing.add(self.drawing.rect(insert=(0, 0), size=('100%', '100%'),
                                           rx=None, ry=None, fill=default_background_color))

        current_line = 0

        for column in tqdm(range(self.number_of_columns), total=self.number_of_columns, desc="Generating columns"):
            x = math.ceil(column * self.column_width)
            x_with_gap = x if column == 0 else x + self.column_gap_size

            self.drawing.add(self.drawing.rect(insert=(x_with_gap, 0), size=(self.column_width, '100%'),
                                               rx=None, ry=None, fill=default_background_color))

            if column != 0:
                self.drawing.add(self.drawing.rect(insert=(x, 0), size=(self.column_gap_size, '100%'),
                                                   rx=None, ry=None, fill=self.column_gap_color))

            y = 0

            first_line = math.ceil(column * self.text_lines_per_column)

            last_line = min(
                math.ceil((column + 1) * self.text_lines_per_column), len(self.text_lines) - 1)

            if first_line >= len(self.text_lines):
                break

            self.render_microprint_column(
                first_line=first_line, last_line=last_line, x_with_gap=x_with_gap, y=y, current_line=current_line)

            current_line += self.text_lines_per_column

        self.drawing.save()

        logging.info(f"Microprint saved as '{self.output_filename}'")
